package proxy.packet;

import com.google.common.collect.Lists;

import java.io.IOException;
import java.util.Collection;
import java.util.List;
import java.util.UUID;

import proxy.Proxy;
import proxy.handler.ProxyHandler;
import proxy.network.Packet;
import proxy.network.PacketBuffer;

public class S38PacketPlayerListItem implements Packet<ProxyHandler> {
	public static enum Action {
		ADD_PLAYER, UPDATE_GAME_MODE, UPDATE_LATENCY, UPDATE_DISPLAY_NAME, REMOVE_PLAYER;
	}

	private class PlayerInfo {
		private final String name;
		private final int ping;
		private final int mode;
		private final String displayName;
		private final String textureData;
		private final String textureSignature;

		public PlayerInfo(String name, int ping, int mode, String display, String texture, String signature) {
			this.name = name;
			this.ping = ping;
			this.mode = mode;
			this.displayName = display;
			this.textureData = texture;
			this.textureSignature = signature;
		}

		public String getName() {
			return this.name;
		}

		public int getPing() {
			return this.ping;
		}

		public int getMode() {
			return this.mode;
		}

		public String getDisplayName() {
			return this.displayName;
		}

		public String getTextureData() {
			return this.textureData;
		}

		public String getTextureSignature() {
			return this.textureSignature;
		}
	}

	private final List<PlayerInfo> players;

	private Action action;

	public S38PacketPlayerListItem() {
		this.players = Lists.newArrayList();
	}

	public S38PacketPlayerListItem(Collection<ProxyHandler> players, boolean pingOnly) {
		this.players = Lists.newArrayListWithCapacity(players.size());
		for(ProxyHandler player : players) {
			this.players.add(new PlayerInfo(player.getUsername(), player.getPing(), player.getMode(), player.getListDisplayName(), player.getTextureData(), player.getTextureSignature()));
		}
		this.action = pingOnly ? Action.UPDATE_LATENCY : Action.ADD_PLAYER;
	}

	public S38PacketPlayerListItem(ProxyHandler player, Action action) {
		this.players = Lists.newArrayList(new PlayerInfo(player.getUsername(), player.getPing(), player.getMode(), player.getListDisplayName(), player.getTextureData(), player.getTextureSignature()));
		this.action = action;
	}

	public void readPacketData(PacketBuffer buf) throws IOException {
		this.action = buf.readEnumValue(Action.class);
		int i = buf.readVarIntFromBuffer();

		for(int j = 0; j < i; ++j) {
			switch(this.action) {
			case ADD_PLAYER:
				buf.readUuid();
				buf.readStringFromBuffer(16);
				int l = buf.readVarIntFromBuffer();

				for(int i1 = 0; i1 < l; ++i1) {
					String s = buf.readStringFromBuffer(32767);
					String s1 = buf.readStringFromBuffer(32767);
					if(buf.readBoolean())
						buf.readStringFromBuffer(32767);
				}

				buf.readVarIntFromBuffer();
				buf.readVarIntFromBuffer();

				if(buf.readBoolean())
					buf.readChatComponent();

				break;

			case UPDATE_GAME_MODE:
				buf.readUuid();
				buf.readVarIntFromBuffer();
				break;

			case UPDATE_LATENCY:
				buf.readUuid();
				buf.readVarIntFromBuffer();
				break;

			case UPDATE_DISPLAY_NAME:
				buf.readUuid();

				if(buf.readBoolean())
					buf.readChatComponent();

				break;

			case REMOVE_PLAYER:
				buf.readUuid();
			}
		}
	}

	public void writePacketData(PacketBuffer buf) throws IOException {
		buf.writeEnumValue(this.action);
		buf.writeVarIntToBuffer(this.players.size());

		for(PlayerInfo data : this.players) {
			UUID id = Proxy.getOfflineUUID(data.getName());
			switch(this.action) {
			case ADD_PLAYER:
				buf.writeUuid(id);
				buf.writeString(data.getName());
				if(data.getTextureData() == null || data.getTextureSignature() == null) {
					buf.writeVarIntToBuffer(0);
				}
				else {
					buf.writeVarIntToBuffer(1);
					buf.writeString("textures");
					buf.writeString(data.getTextureData());
					buf.writeBoolean(true);
					buf.writeString(data.getTextureSignature());
				}

				buf.writeVarIntToBuffer(data.getMode());
				buf.writeVarIntToBuffer(data.getPing());

				if(data.getDisplayName() == null) {
					buf.writeBoolean(false);
				}
				else {
					buf.writeBoolean(true);
					buf.writeChatComponent(data.getDisplayName());
				}

				break;

			case UPDATE_GAME_MODE:
				buf.writeUuid(id);
				buf.writeVarIntToBuffer(data.getMode());
				break;

			case UPDATE_LATENCY:
				buf.writeUuid(id);
				buf.writeVarIntToBuffer(data.getPing());
				break;

			case UPDATE_DISPLAY_NAME:
				buf.writeUuid(id);

				if(data.getDisplayName() == null) {
					buf.writeBoolean(false);
				}
				else {
					buf.writeBoolean(true);
					buf.writeChatComponent(data.getDisplayName());
				}

				break;

			case REMOVE_PLAYER:
				buf.writeUuid(id);
			}
		}
	}

	public void processPacket(ProxyHandler handler) {
	}
}
